"use strict";

// utilFileMgr
// Wrapper that normalizes cordova file plugin, and chrome filesystem (both implementations of defunct html5 File API).
//--------------
function utilFileMgr() {
	window.requestFileSystem = window.requestFileSystem || window.webkitRequestFileSystem || window.mozRequestFileSystem;  
	
	this._fs = null;
	this._root = null;
	
	this.cordovaFileRoot = null; /* override outside, before initFileSys() call */
		//this.cordovaFileRoot_android = cordova.file.externalRootDirectory;
		//this.cordovaFileRoot_ios = cordova.file.documentsDirectory;

	this.chromeStorageType = window.TEMPORARY; /*or, window.PERSISTENT, override outside, before initFileSys() call.*/
	this.chromeRequestQuotaBytes = 50*1024*1024; /*50MB*/
}


// Partly from:
// - https://github.com/apache/cordova-plugin-file/blob/master/README.md
// - http://www.html5rocks.com/en/tutorials/file/filesystem/ (great tutorial)
// - https://www.neontribe.co.uk/cordova-file-plugin-examples/ (file API enlightening opinions)
utilFileMgr.prototype.initFileSys = function(callback) {
	var self = this;
	/*if (window.cordova) { // SHOULD: pass in rootDir as parameter
		if (g_isIOS && )
			this.cordovaFileRoot = cordova.file.documentsDirectory;
		else
			this.cordovaFileRoot = cordova.file.externalRootDirectory;
	}*/

	// Request FS (unnecessary in cordova.plugin.file)
	//window.requestFileSystem(window.PERSISTENT, 50*1024*1024 /*50MB*/, self._onInitFs, utilFileMgr._onErr);

	// If running under cordova/phonegap (i.e. Android as webrtc unsupported on iOS) - save files in local data dir.
	if (window.cordova) {
		if (!this.cordovaFileRoot)
			alert("Error: fileRoot_cordova unspecified.");
		//g_genUI.TransientMsg("initFS", "Initializing FS - app");
		//alert("cordova FS unimplemented.");
		//window.resolveLocalFileSystemURL(cordova.file.dataDirectory, function (dirEntry) {
		window.resolveLocalFileSystemURL(this.cordovaFileRoot, function (dirEntry) {
			self._root = dirEntry;
			self._onInitFs(callback);
		}, utilFileMgr._onErr);
	}
	// Else, running on PC or browser (chrome).
	else {
		//g_genUI.TransientMsg("initFS", "Initializing FS - browser");
		// Request FS quota (unnecessary in cordova.plugin.file)
		//window.webkitStorageInfo.requestQuota(
		var webkitStorageObj = self.chromeStorageType == window.PERSISTENT ? navigator.webkitPersistentStorage : navigator.webkitTemporaryStorage;
		webkitStorageObj.requestQuota(
			self.chromeRequestQuotaBytes, 
			function (grantedBytes) {  
				// Request FS (unnecessary in cordova.plugin.file)
				window.requestFileSystem(self.chromeStorageType, grantedBytes, 
					function(fs) { self._root = fs.root; self._onInitFs(callback); }, 
					utilFileMgr._onErr
				);
			}, function (e) {
				console.log('Error', e);
			}
		);
	}
}


utilFileMgr.prototype._onInitFs = function(callback) {
	//this._fs = fs;
	
	// Create photos folder (and test file in root).
	// TODO: decide if we should do that here???
	
	if (callback)
		callback();
	//console.log('Opened file system: ' + fs.name);
}

utilFileMgr._onErr = function(e) {
    var msg = '';

	//switch (e.code) {
	/*switch (e.message) {
		case FileError.QUOTA_EXCEEDED_ERR:
			msg = 'Storage quota exceeded';
			break;
		case FileError.NOT_FOUND_ERR:
			msg = 'File not found';
			break;
		case FileError.SECURITY_ERR:
			msg = 'Security error';
			break;
		case FileError.INVALID_MODIFICATION_ERR:
			msg = 'Invalid modification';
			break;
		case FileError.INVALID_STATE_ERR:
			msg = 'Invalid state';
			break;
		default:
			msg = 'Unknown error';
			break;
	};*/
	
    //console.log('FS error (' + fileName + '): ' + msg);
    //alert('FS error (' + fileName + '): ' + msg);
    console.log('FS error: ' + msg + "/" + e.name + "/" + e.message);
    alert('FS error: ' + msg + "/" + e.name + "/" + e.message);
}


utilFileMgr.prototype.createDir  = function(dirPath, callback) {
	this._root.getDirectory(dirPath, {create: true}, function(dirEntry) {
		if (callback)
			callback(dirEntry);
	}, utilFileMgr._onErr);
}

utilFileMgr.prototype.getFile = function(filePath, create, callback) {
	this._root.getFile(filePath, {create: create}, function(fileEntry) {
		if (callback)
			callback(fileEntry);
	}, utilFileMgr._onErr);
}

utilFileMgr.prototype.fileExists = function(filePath, callback) {
	this._root.getFile(filePath, {create : false}, function() {
		callback(true);
	}, function() {
		callback(false);
	});
}
utilFileMgr.prototype.dirExists = function(dirPath, callback) {
	this._root.getDirectory(dirPath, {create : false}, function() {
		callback(true);
	}, function() {
		callback(false);
	});
}

// Read files from sub-dir
// - Creates dir if DNE.
utilFileMgr.prototype.dirContents  = function(dirPath, callback) {
	//var dirReader = this._root.createReader();
	//var entries = [];
  
	// Get photos directory
	this._root.getDirectory(dirPath, {create: true}, function(dirEntry) {
		var dirReader = dirEntry.createReader();
		dirReader.readEntries (
			function(results) {
				/*if (!results.length) {
					listResults(entries.sort());
				} else {
					entries = entries.concat(toArray(results));
					readEntries();
				}*/
				//callback(toArray(results));
				//console.log(results);
				callback(results);
			}, utilFileMgr._onErr
		);
	});
}

utilFileMgr.prototype.readFileAsDataUrl = function(filePath, callback) {
	this._root.getFile(filePath, {create: false}, function(fileEntry) {
		fileEntry.file(function(file) {
			var reader = new FileReader();
			reader.onloadend = function(e) {
				callback(this.result);
			};
			//reader.readAsText(file);
			reader.readAsDataURL(file);
		}, utilFileMgr._onErr);
	}, utilFileMgr._onErr);
}

utilFileMgr.prototype.readFileEntryAsText = function(fileEntry, callback) {
	fileEntry.file(function(file) {
		var reader = new FileReader();
		reader.onloadend = function(e) {
			callback(this.result);
		};
		reader.readAsText(file);
	}, utilFileMgr._onErr);
}


// Misnomer - creates any file (e.g. text file if blob is JS string).
utilFileMgr.prototype.createJpegFile  = function(dirPath, filename, blob, callback) {
	//console.log("1");
	this._root.getDirectory(dirPath, {create: true}, function(dirEntry) {
		//console.log("2");
		
		if (typeof blob === "string")
		blob = new Blob([blob], {type: 'text/plain'});
				
		dirEntry.getFile(filename, {create: true}, function(fileEntry) {
			//console.log("3");
			// Create a FileWriter object for our FileEntry (log.txt).
			fileEntry.createWriter(function(fileWriter) {
				//console.log("4");

				fileWriter.onerror = utilFileMgr._onErr;

				fileWriter.onwriteend = function(e) {
					fileWriter.onwriteend = null; // Avoid an infinite loop, from truncate(1) call below.
					//e.currentTarget.truncate(e.currentTarget.position); // To fix overwriting a smaller file over top of a larger file (remove larger file contents in excess of smaller file's size).
					
					// Truncate, for writing shorter files over longer files. http://stackoverflow.com/a/6868034
					//fileWriter.seek(fileWriter.length); // Start write position at EOF.
					//fileWriter.truncate(fileWriter.length);
					fileWriter.truncate(blob.size);

					
					//console.log("5");
					console.log('Write completed - ' + filename);
					if (callback)
						callback();
				};
				//fileWriter.write(blobBuilder.getBlob("image/jpeg"));
				fileWriter.write(blob);
				
			}, utilFileMgr._onErr);
		}, utilFileMgr._onErr);
	}, utilFileMgr._onErr);
}

utilFileMgr.prototype.deleteFile  = function(filePath, callback) {
	console.log("deleting " + filePath);
	this._root.getFile(filePath, {create: false}, function(fileEntry) {
		//console.log("deleting2 " + filePath);
		fileEntry.remove(function() {
			//console.log("deleted " + filePath);
			callback();
		}, utilFileMgr._onErr);
	}, utilFileMgr._onErr);
}
utilFileMgr.prototype.deleteDir = function(dirPath, callback) {
	console.log("deleting " + dirPath);
	this._root.getDirectory(dirPath, {create: false}, function(dirEntry) {
		dirEntry.remove(function() {
			callback();
		}, utilFileMgr._onErr);
	}, utilFileMgr._onErr);
}

utilFileMgr.prototype.moveFile = function(srcFilePath, destDirPath, destFileName, callback) {
	var self = this;
	//console.log("move 1 - " + srcFilePath);
	self.getFile(srcFilePath, false, function(fileEntry) {
	//console.log("move 2");
		self._root.getDirectory(destDirPath, {create: false}, function(destDirEntry) {
	//console.log("move 3");
			fileEntry.moveTo(destDirEntry, destFileName, function() {
	//console.log("move 4");
				callback();
			}, utilFileMgr._onErr);
		}, utilFileMgr._onErr);
	}, utilFileMgr._onErr);
}



// Move absolute-file-path into directory within this FS.
utilFileMgr.prototype.moveAbsFileIn = function(sourceFileAbsPath, destDirFsPath, destFilename, callback) {
	var self = this;
	if (!window.cordova) {
		alert("cordova required for moveAbsFileIn()");
		return;
	}
	window.resolveLocalFileSystemURL(sourceFileAbsPath, function (fileEntry) {
		self._root.getDirectory(destDirFsPath, {create: true}, function(dirEntry) {
			fileEntry.moveTo(dirEntry, destFilename,
				function() { callback(fileEntry); }, 
				utilFileMgr._onErr
			);
		});
	}, utilFileMgr._onErr);
}
