"use strict";

// Show uncaught exceptions to user.
window.onerror = function(msg, url, line, col, error) {
	var msgFull = "Error: " + msg + "\nurl: " + url + "\nline: " + line;
	alert(msgFull);
};

window.g_getUrlParam = function(name) {
	name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");
	var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
		results = regex.exec(location.search);
	return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
}

window.g_htmlEncode = function(html) {
	return document.createElement('a').appendChild( 
		document.createTextNode(html)).parentNode.innerHTML;
}

window.g_setHash = function(hashVal) {
	if (appWindow.location.hash == hashVal)
		$(appWindow).trigger("hashchange"); // trigger page refresh if navigating to same page.
	else
		appWindow.location.hash = hashVal;
}

window.g_openUrl = function(url, target) {
	if (!url)
		throw "Error: no link url.";
	if (window.cordova)
		cordova.InAppBrowser.open(url, target, ""); // system browser
	else
		window.open(url, "_system", "height=500,width=500");
}


// Call initially, and when window is resized.
window.g_onWindowResize = function() {
	// Set hight to fill offsetParent element.
	if ($(".jsHeightToFull").length) {
		$(".jsHeightToFull").each(function() {
			var $thisEl = $(this);
			var h = $thisEl.offsetParent().innerHeight() - $thisEl.position().top;
			// If not no positioned ancestor, go to full window height.
			if ($thisEl.offsetParent()[0].tagName.toLowerCase() == "html")
				h = $(window).innerHeight() - $thisEl.offset().top;
			$thisEl.height(h);
		});
	}
}



// If in an iframe, assume parent has been setup, and import stuff.
//-----------------------------------------------------------------
var appWindow = window;
if (window.parent && window.parent.g_genUI) {
	// Import shared global objects from parent app into this iframe.
	window.cordova = window.parent.cordova;
	window.FastClick = window.parent.FastClick;
	
	window.g_isIOS = window.parent.g_isIOS;
	window.g_genUI = window.parent.g_genUI;
	window.g_stateMgr = window.parent.g_stateMgr;
	window.g_pageSwapper = window.parent.g_pageSwapper;

	appWindow = window.parent.window;

	// Add iOS statusbar padding if root frame has it.
	if (window.parent.$("body").hasClass("iOSPaddingEnabled"))
		$("body").addClass("iOSPaddingEnabled");
}


// Wireup universal event handlers.
//---------------------------------
$(document).ready(function() {
	// Remove 300ms click delay (in some old versions of chrome, and possibly iOS)
	if(window.FastClick)
		FastClick.attach(document.body);

	// Wireup jsHeightToBottom element.
	setTimeout(g_onWindowResize, 50);
	$(window).on("resize", function() {
		g_onWindowResize();
	});

	
	// url-open handler
	$("body").on("click", ".jsOpenExt", function() {
		var url = $(this).data("href");
		g_openUrl(url, "_system"); // system browser
	});
	$("body").on("click", ".jsOpenIAB", function() {
		var url = $(this).data("href");
		g_openUrl(url, "_blank"); // IAB browser
	});

	// Frame navigation
	$("body").on("click", ".jsNavToPage", function(e) {
		e.stopPropagation();
		var $thisBtn = $(this);
		$thisBtn.addClass("clicked");
		var srcUrl = $(this).data("srcurl");
		//console.log("nav to page: " + srcUrl);
		setTimeout(function() {
			//g_pageSwapper.swapToFrame(srcUrl);
			g_setHash("#" + srcUrl);
		}, 10);
	});

	// Overlay closer
	$("body").on("click", ".jsNavHome", function() {
		//g_pageSwapper.hideAllOverlays();
		g_setHash("#home");
	});
	
	// Clicked style
	$("body").on("click", ".jsClickable", function() {
		var $thisBtn = $(this);
		$thisBtn.addClass("clicked");
	});
	
});
